/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef FILEHOLDER_H_INCLUDED
#define FILEHOLDER_H_INCLUDED

#include <c4scripter/c4s-documentwindow.h>
#include <c4scripter/c4s-document.h>
#include <c4scripter/c4s-enginefunctionlist.h>
#include <c4scripter/c4s-keywordlist.h>
#include <c4scripter/c4s-preferences.h>

#include <gtkmm/notebook.h>

class FileHolder: public Gtk::Notebook
{
public:
  typedef sigc::signal<void, C4sDocumentWindow*> SignalDocumentAdded;
  typedef sigc::signal<void, C4sDocumentWindow*> SignalDocumentChanged;
  typedef sigc::signal<void, C4sDocumentWindow*> SignalDocumentRemoved;

  typedef sigc::signal<void, C4sDocumentWindow*> SignalDocumentClose;

  FileHolder(C4sPreferences* preferences, Gtk::Window& parent);
  ~FileHolder();

  C4sDocumentWindow* create_document();
  C4sDocumentWindow* create_document(const std::string& path);
  void remove_document(C4sDocumentWindow* window);

  C4sDocumentWindow* get_current_document();
  
  C4sEngineFunctionList* get_engine_function_list() { return m_functions; }

  SignalDocumentChanged signal_document_changed() const { return m_signal_document_changed; }
  SignalDocumentAdded signal_document_added() const { return m_signal_document_added; }
  SignalDocumentRemoved signal_document_removed() const { return m_signal_document_removed; }

  // Is emitted when the user wants to close a document. Signal handlers may
  // decide to call remove_document upon signal emission or prompt a close
  // confirmation first or whatever.
  SignalDocumentClose signal_document_close() const { return m_signal_document_close; }
private:
  virtual void on_switch_page(GtkNotebookPage* page, guint page_num);

  void on_document_close(C4sDocumentWindow* window);

  C4sDocumentWindow* create_document(C4sDocumentWindow* window);

  C4sPreferences* m_preferences;
  Gtk::Window& m_parent;

  C4sKeywordList* m_keywords;
  C4sEngineFunctionList* m_functions;

  SignalDocumentChanged m_signal_document_changed;
  SignalDocumentAdded m_signal_document_added;
  SignalDocumentRemoved m_signal_document_removed;
  SignalDocumentClose m_signal_document_close;
};

#endif // FILEHOLDER_H_INCLUDED
