/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __C4S_CUSTOM_FUNCTION_LIST_H__
#define __C4S_CUSTOM_FUNCTION_LIST_H__

#include <c4scripter/c4s-customfunction.h>

#include <glib-object.h>

G_BEGIN_DECLS

#define C4S_TYPE_CUSTOM_FUNCTION_LIST              (c4s_custom_function_list_get_type())
#define C4S_CUSTOM_FUNCTION_LIST(obj)              (G_TYPE_CHECK_INSTANCE_CAST((obj), C4S_TYPE_CUSTOM_FUNCTION_LIST, C4sCustomFunctionList))
#define C4S_CUSTOM_FUNCTION_LIST_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST((klass), C4S_TYPE_CUSTOM_FUNCTION_LIST, C4sCustomFunctionListClass))
#define C4S_IS_CUSTOM_FUNCTION_LIST(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), C4S_TYPE_CUSTOM_FUNCTION_LIST))
#define C4S_IS_CUSTOM_FUNCTION_LIST_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE((klass), C4S_TYPE_CUSTOM_FUNCTION_LIST))
#define C4S_CUSTOM_FUNCTION_LIST_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS((obj), C4S_TYPE_CUSTOM_FUNCTION_LIST, C4sCustomFunctionListClass))

typedef struct _C4sCustomFunctionListClass C4sCustomFunctionListClass;
typedef struct _C4sCustomFunctionList C4sCustomFunctionList;

typedef void(*C4sCustomFunctionListForeachFunc)(C4sCustomFunction*, gpointer);

struct _C4sCustomFunctionListClass
{
  GObjectClass parent_class;
  
  void(*function_added)(C4sCustomFunctionList*, C4sCustomFunction*);
  void(*function_removed)(C4sCustomFunctionList*, C4sCustomFunction*);
};

struct _C4sCustomFunctionList
{
  GObject parent_instance;
};

GType
c4s_custom_function_list_get_type(void) G_GNUC_CONST;

C4sCustomFunctionList*
c4s_custom_function_list_new(GtkTextBuffer* buffer,
                             C4sHighlighter* highlighter);

/* This function does not find functions whose name-changed signal is
 * currently being emitted. It also finds only the first function matching
 * the given name if there are more than one. */
C4sCustomFunction*
c4s_custom_function_list_find_function(const C4sCustomFunctionList* list,
                                       const gchar* name);

void
c4s_custom_function_list_foreach(C4sCustomFunctionList* list,
                                 C4sCustomFunctionListForeachFunc func,
                                 gpointer user_data);

G_END_DECLS

#endif /* __C4S_CUSTOM_FUNCTION_LIST_H__ */
