/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __C4S_DOCUMENT_H__
#define __C4S_DOCUMENT_H__

#include <c4scripter/c4s-paraminfo.h>
#include <c4scripter/c4s-customfunctionlist.h>
#include <c4scripter/c4s-enginefunctionlist.h>
#include <c4scripter/c4s-preferences.h>
#include <c4scripter/c4s-keywordlist.h>

#include <gtksourceview/gtksourcebuffer.h>

G_BEGIN_DECLS

#define C4S_TYPE_DOCUMENT                   (c4s_document_get_type())
#define C4S_DOCUMENT(obj)                   (G_TYPE_CHECK_INSTANCE_CAST((obj), C4S_TYPE_DOCUMENT, C4sDocument))
#define C4S_DOCUMENT_CLASS(klass)           (G_TYPE_CHECK_CLASS_CAST((klass), C4S_TYPE_DOCUMENT, C4sDocumentClass))
#define C4S_IS_DOCUMENT(obj)                (G_TYPE_CHECK_INSTANCE_TYPE ((obj), C4S_TYPE_DOCUMENT))
#define C4S_IS_DOCUMENT_CLASS(klass)        (G_TYPE_CHECK_CLASS_TYPE((klass), C4S_TYPE_DOCUMENT))
#define C4S_DOCUMENT_GET_CLASS(obj)         (G_TYPE_INSTANCE_GET_CLASS((obj), C4S_TYPE_DOCUMENT, C4sDocumentClass))

#define C4S_TYPE_DOCUMENT_LINE_ENDINGS      (c4s_document_line_endings_get_type())

typedef struct _C4sDocumentClass C4sDocumentClass;
typedef struct _C4sDocument C4sDocument;

typedef enum _C4sDocumentLineEndings {
  C4S_DOCUMENT_LINE_ENDINGS_DOS,
  C4S_DOCUMENT_LINE_ENDINGS_UNIX,
  C4S_DOCUMENT_LINE_ENDINGS_MAC
} C4sDocumentLineEndings;

struct _C4sDocumentClass
{
  GtkSourceBufferClass parent_class;

  void(*title_changed)(C4sDocument*, const gchar*);
  void(*path_changed)(C4sDocument*, const gchar*);
};

struct _C4sDocument
{
  GtkSourceBuffer parent_instance;
};

GType
c4s_document_get_type(void) G_GNUC_CONST;

GType
c4s_document_line_endings_get_type(void) G_GNUC_CONST;

C4sDocument*
c4s_document_new(C4sPreferences* preferences);

gboolean
c4s_document_load_file(C4sDocument* document,
                       const gchar* filename,
                       GError** error);

gboolean
c4s_document_save_file(C4sDocument* document,
                       const gchar* filename,
                       GError** error);

const gchar*
c4s_document_get_title(C4sDocument* document);

const gchar*
c4s_document_get_path(C4sDocument* document);

C4sCustomFunctionList*
c4s_document_get_custom_function_list(C4sDocument* document);

C4sParamInfo*
c4s_document_get_param_info(C4sDocument* document);

G_END_DECLS

#endif /* __C4S_DOCUMENT_H__ */
