/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __C4S_ENGINEFUNCTIONLIST_H__
#define __C4S_ENGINEFUNCTIONLIST_H__

#include <c4scripter/c4s-enginefunction.h>
#include <c4scripter/c4s-preferences.h>

#include <gtk/gtkwindow.h>
#include <gtk/gtkliststore.h>

#include <glib-object.h>

G_BEGIN_DECLS

#define C4S_TYPE_ENGINE_FUNCTION_LIST			(c4s_engine_function_list_get_type())
#define C4S_ENGINE_FUNCTION_LIST(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), C4S_TYPE_ENGINE_FUNCTION_LIST, C4sEngineFunctionList))
#define C4S_ENGINE_FUNCTION_LIST_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), C4S_TYPE_ENGINE_FUNCTION_LIST, C4sEngineFunctionListClass))
#define C4S_IS_ENGINE_FUNCTION_LIST(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), C4S_TYPE_ENGINE_FUNCTION_LIST))
#define C4S_IS_ENGINE_FUNCTION_LIST_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), C4S_TYPE_ENGINE_FUNCTION_LIST))
#define C4S_ENGINE_FUNCTION_LIST_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), C4S_TYPE_ENGINE_FUNCTION_LIST, C4sEngineFunctionListClass))

typedef enum _C4sLoaderError {
	C4S_LOADER_TRANSPORT_ERROR,
	C4S_LOADER_PARSE_ERROR
} C4sLoaderError;

typedef struct _C4sEngineFunctionListClass C4sEngineFunctionListClass;
typedef struct _C4sEngineFunctionList C4sEngineFunctionList;

typedef struct _C4sLoaderBackend C4sLoaderBackend;

typedef struct _C4sLoaderBackendCallbacks C4sLoaderBackendCallbacks;
struct _C4sLoaderBackendCallbacks {
  void(*init_done)(unsigned int, gpointer);
  void(*init_failed)(const GError*, gpointer);
  void(*function_done)(C4sEngineFunction*, gpointer);
  void(*function_failed)(const gchar*, const GError*, gpointer);
  void(*set_status)(const gchar*, gpointer);
};

typedef struct _C4sLoaderBackendFuncs C4sLoaderBackendFuncs;
struct _C4sLoaderBackendFuncs {
  C4sLoaderBackend*(*init)(
    C4sPreferences*,
    const C4sLoaderBackendCallbacks*,
    gpointer
  );

  void(*deinit)(C4sLoaderBackend*);

  void(*load_function)(C4sLoaderBackend*, unsigned int i);
};

struct _C4sEngineFunctionListClass
{
  GObjectClass parent_class;

  void(*reloaded)(C4sEngineFunctionList*);
};

struct _C4sEngineFunctionList
{
  GObject parent_instance;
};

GType
c4s_engine_function_list_get_type(void) G_GNUC_CONST;

C4sEngineFunctionList*
c4s_engine_function_list_new(GtkWindow* parent,
                             C4sPreferences* preferences);

const C4sEngineFunction*
c4s_engine_function_list_find_function(const C4sEngineFunctionList* list,
                                       const gchar* name);

GtkListStore*
c4s_engine_function_list_get_list_store(const C4sEngineFunctionList* list);

void
c4s_engine_function_list_reload(C4sEngineFunctionList* list);

G_END_DECLS

#endif /* __C4S_ENGINEFUNCTIONLIST_H__ */
