/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __C4S_FUNCTION_H__
#define __C4S_FUNCTION_H__

#include <c4scripter/c4s-valuetype.h>

#include <glib-object.h>

G_BEGIN_DECLS

#define C4S_TYPE_FUNCTION             (c4s_function_get_type())
#define C4S_FUNCTION(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj), C4S_TYPE_FUNCTION, C4sFunction))
#define C4S_FUNCTION_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST((klass), C4S_TYPE_FUNCTION, C4sFunctionClass))
#define C4S_IS_FUNCTION(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), C4S_TYPE_FUNCTION))
#define C4S_IS_FUNCTION_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE((klass), C4S_TYPE_FUNCTION))
#define C4S_FUNCTION_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS((obj), C4S_TYPE_FUNCTION, C4sFunctionClass))

typedef struct _C4sFunctionClass C4sFunctionClass;
typedef struct _C4sFunction C4sFunction;

typedef struct _C4sFunctionParameter C4sFunctionParameter;
struct _C4sFunctionParameter
{
  C4sValueType type;
  gchar* name;
};

struct _C4sFunctionClass
{
  GObjectClass parent_class;
};

struct _C4sFunction
{
  GObject parent_instance;
};

GType
c4s_function_get_type(void) G_GNUC_CONST;

C4sValueType
c4s_function_get_return_type(const C4sFunction* function);

const C4sFunctionParameter*
c4s_function_get_parameters(const C4sFunction* function,
                            gsize* n_parameters);

const gchar*
c4s_function_get_name(const C4sFunction* function);

/* semi-public */
void
c4s_function_add_parameter(C4sFunction* function,
                           C4sValueType type,
                           const gchar* name);

/* semi-public */
void
c4s_function_clear_parameters(C4sFunction* function);

G_END_DECLS

#endif /* __C4S_FUNCTION_H__ */
