/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <c4scripter/c4s-functionprogressdialog.h>
#include <c4scripter/c4s-i18n.h>

#include <gtk/gtkprogressbar.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkmisc.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkbox.h>
#include <gtk/gtkstock.h>

typedef struct _C4sFunctionProgressDialogPrivate C4sFunctionProgressDialogPrivate;
struct _C4sFunctionProgressDialogPrivate {
  GtkWidget* progress;
};

#define C4S_FUNCTION_PROGRESS_DIALOG_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE( \
    (o), \
    C4S_TYPE_FUNCTION_PROGRESS_DIALOG, \
    C4sFunctionProgressDialogPrivate \
  ))

static GtkDialogClass* parent_class = NULL;

static void
c4s_function_progress_dialog_init(C4sFunctionProgressDialog* dialog)
{
  GtkWidget* label;
  GtkWidget* vbox;
  C4sFunctionProgressDialogPrivate* priv;

  priv = C4S_FUNCTION_PROGRESS_DIALOG_PRIVATE(dialog);
  
  label = gtk_label_new(
    _("Function list is being built, please stand by..."));
  gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
  gtk_widget_show(label);
  
  priv->progress = gtk_progress_bar_new();
  gtk_widget_show(priv->progress);

  vbox = gtk_vbox_new(FALSE, 8);

  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), priv->progress, FALSE, FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
  gtk_widget_show(vbox);
  
  gtk_box_pack_start(
    GTK_BOX(GTK_DIALOG(dialog)->vbox),
    vbox,
    FALSE,
    FALSE,
    0
  );

  gtk_dialog_add_button(
    GTK_DIALOG(dialog),
    GTK_STOCK_CANCEL,
    GTK_RESPONSE_CANCEL
  );

  gtk_window_set_title(GTK_WINDOW(dialog), _("Function database progress"));
}

static void 
c4s_function_progress_dialog_finalize(GObject* object)
{
  C4sFunctionProgressDialog* function_progress_dialog;
  C4sFunctionProgressDialogPrivate* priv;

  function_progress_dialog = C4S_FUNCTION_PROGRESS_DIALOG(object);
  priv = C4S_FUNCTION_PROGRESS_DIALOG_PRIVATE(function_progress_dialog);

  G_OBJECT_CLASS(parent_class)->finalize(object);
}

static void
c4s_function_progress_dialog_class_init(C4sFunctionProgressDialogClass* klass)
{
  GObjectClass* object_class = G_OBJECT_CLASS(klass);
  parent_class = GTK_DIALOG_CLASS(g_type_class_peek_parent(klass));

  g_type_class_add_private(klass, sizeof(C4sFunctionProgressDialogPrivate));

  object_class->finalize = c4s_function_progress_dialog_finalize;
}

GType
c4s_function_progress_dialog_get_type(void)
{
  static GType our_type = 0;

  if(our_type == 0)
  {
    static const GTypeInfo our_info = {
      sizeof(C4sFunctionProgressDialogClass),
      (GBaseInitFunc)NULL,
      (GBaseFinalizeFunc)NULL,
      (GClassInitFunc)c4s_function_progress_dialog_class_init,
      NULL,
      NULL,
      sizeof(C4sFunctionProgressDialog),
      0,
      (GInstanceInitFunc)c4s_function_progress_dialog_init
    };

    our_type = g_type_register_static(
      GTK_TYPE_DIALOG,
      "C4sFunctionProgressDialog",
      &our_info,
      0
    );
  }

  return our_type;
}

C4sFunctionProgressDialog*
c4s_function_progress_dialog_new(GtkWindow* parent,
                                 GtkDialogFlags flags)
{
  GObject* object;

  object = g_object_new(
    C4S_TYPE_FUNCTION_PROGRESS_DIALOG,
    NULL
  );

  if(flags & GTK_DIALOG_MODAL)
    gtk_window_set_modal(GTK_WINDOW(object), TRUE);

  if(flags & GTK_DIALOG_DESTROY_WITH_PARENT)
    gtk_window_set_destroy_with_parent(GTK_WINDOW(object), TRUE);

  if(flags & GTK_DIALOG_NO_SEPARATOR)
    gtk_dialog_set_has_separator(GTK_DIALOG(object), FALSE);

  gtk_window_set_transient_for(GTK_WINDOW(object), parent);
  return C4S_FUNCTION_PROGRESS_DIALOG(object);
}

void
c4s_function_progress_dialog_set_progress(C4sFunctionProgressDialog* dialog,
                                          double progress)
{
  char prc_text[16];

  gtk_progress_bar_set_fraction(
    GTK_PROGRESS_BAR(C4S_FUNCTION_PROGRESS_DIALOG_PRIVATE(dialog)->progress),
    progress
  );

  sprintf(prc_text, "%u %%", (unsigned int)(progress * 100.0));
  gtk_progress_bar_set_text(
    GTK_PROGRESS_BAR(C4S_FUNCTION_PROGRESS_DIALOG_PRIVATE(dialog)->progress),
    prc_text
  );
}

void
c4s_function_progress_dialog_set_status(C4sFunctionProgressDialog* dialog,
                                        const char* status)
{
  gtk_progress_bar_set_text(
    GTK_PROGRESS_BAR(C4S_FUNCTION_PROGRESS_DIALOG_PRIVATE(dialog)->progress),
    status
  );
}
