/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <c4scripter/c4s-functionquerywidget.h>
#include <c4scripter/c4s-marshal.h>
#include <c4scripter/c4s-i18n.h>

#include <gtk/gtkfilechooserbutton.h>
#include <gtk/gtkfilechooserdialog.h>
#include <gtk/gtkfilechooser.h>
#include <gtk/gtkradiobutton.h>
#include <gtk/gtkalignment.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkstock.h>

typedef struct _C4sFunctionQueryWidgetPrivate C4sFunctionQueryWidgetPrivate;
struct _C4sFunctionQueryWidgetPrivate {
  GtkWidget* radio_chm;
  GtkWidget* radio_online;

  GtkWidget* chm_file_chooser;
};

#define C4S_FUNCTION_QUERY_WIDGET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE( \
    (o), \
    C4S_TYPE_FUNCTION_QUERY_WIDGET, \
    C4sFunctionQueryWidgetPrivate \
  ))

enum {
  PROP_0,

  PROP_ACTION,
  PROP_CHM
};

enum {
  UPDATE,
  
  LAST_SIGNAL
};

static GtkVBoxClass* parent_class = NULL;
static guint function_query_widget_signals[LAST_SIGNAL];

static void
c4s_function_query_widget_update(C4sFunctionQueryWidget* widget)
{
  C4sFunctionQueryWidgetPrivate* priv;
  gchar* filename;

  priv = C4S_FUNCTION_QUERY_WIDGET_PRIVATE(widget);

  gtk_widget_set_sensitive(
    priv->chm_file_chooser,
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(priv->radio_chm))
  );

  filename = gtk_file_chooser_get_filename(
    GTK_FILE_CHOOSER(priv->chm_file_chooser)
  );

  g_signal_emit(
    G_OBJECT(widget),
    function_query_widget_signals[UPDATE],
    0,
    c4s_function_query_widget_get_action(widget),
    filename
  );

  g_free(filename);
}

static void
c4s_function_query_widget_chm_cb(G_GNUC_UNUSED GtkButton* radio_button,
                                 gpointer user_data)
{
  C4sFunctionQueryWidget* widget;
  widget = C4S_FUNCTION_QUERY_WIDGET(user_data);

  c4s_function_query_widget_update(widget);
}

static void
c4s_function_query_widget_selection_changed_cb(G_GNUC_UNUSED
                                               GtkFileChooser* chooser,
                                               gpointer user_data)
{
  C4sFunctionQueryWidget* widget;
  widget = C4S_FUNCTION_QUERY_WIDGET(user_data);
  
  c4s_function_query_widget_update(widget);
}

static void
c4s_function_query_widget_online_cb(G_GNUC_UNUSED GtkButton* radio_button,
                                    gpointer user_data)
{
  C4sFunctionQueryWidget* widget;
  widget = C4S_FUNCTION_QUERY_WIDGET(user_data);

  c4s_function_query_widget_update(widget);
}

static void
c4s_function_query_widget_init(C4sFunctionQueryWidget* widget)
{
  /*GtkWidget* vbox_radio;*/

  GtkFileFilter* filter_chm;
  GtkFileFilter* filter_all;

  GtkWidget* chm_widget;
  GtkWidget* button_alignment;

  C4sFunctionQueryWidgetPrivate* priv;

  priv = C4S_FUNCTION_QUERY_WIDGET_PRIVATE(widget);

  priv->radio_chm = gtk_radio_button_new_with_label(
    NULL,
    _("Get script functions from the following CHM file:")
  );

  gtk_widget_show(priv->radio_chm);

  priv->radio_online = gtk_radio_button_new_with_label(
    gtk_radio_button_get_group(GTK_RADIO_BUTTON(priv->radio_chm)),
    _("Get script functions online from http://clonk.de.")
  );

  /*gtk_widget_show(priv->radio_online);*/

  chm_widget = gtk_file_chooser_dialog_new(
    _("Select clonk documentation CHM file"),
    NULL, /* TODO: Find a window here. hm. GTK_WINDOW(widget), */
    GTK_FILE_CHOOSER_ACTION_OPEN,
    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
    GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
    NULL
  );

  priv->chm_file_chooser = gtk_file_chooser_button_new_with_dialog(
    chm_widget
  );

  gtk_file_chooser_button_set_width_chars(
    GTK_FILE_CHOOSER_BUTTON(priv->chm_file_chooser),
    25
  );

  filter_chm = gtk_file_filter_new();
  gtk_file_filter_set_name(filter_chm, _("CHM files"));
  gtk_file_filter_add_pattern(filter_chm, "*.chm");

  filter_all = gtk_file_filter_new();
  gtk_file_filter_set_name(filter_all, _("All files"));
  gtk_file_filter_add_pattern(filter_all, "*");

  gtk_file_chooser_add_filter(
    GTK_FILE_CHOOSER(priv->chm_file_chooser),
    filter_chm
  );

  gtk_file_chooser_add_filter(
    GTK_FILE_CHOOSER(priv->chm_file_chooser),
    filter_all
  );

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(priv->radio_chm), TRUE);
  /*gtk_widget_set_sensitive(priv->chm_file_chooser, FALSE);*/
  gtk_widget_show(priv->chm_file_chooser);

  button_alignment = gtk_alignment_new(0, 0.5, 0.0, 0.0);
  gtk_alignment_set_padding(GTK_ALIGNMENT(button_alignment), 0, 0, 25, 0);
  gtk_container_add(GTK_CONTAINER(button_alignment), priv->chm_file_chooser);
  gtk_widget_show(button_alignment);

  gtk_box_set_spacing(GTK_BOX(widget), 6);
  gtk_box_pack_start(GTK_BOX(widget), priv->radio_chm, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(widget), button_alignment, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(widget), priv->radio_online, FALSE, FALSE, 0);

  g_signal_connect_after(
    G_OBJECT(priv->radio_chm),
    "toggled",
    G_CALLBACK(c4s_function_query_widget_chm_cb),
    widget
  );

  g_signal_connect_after(
    G_OBJECT(priv->radio_online),
    "toggled",
    G_CALLBACK(c4s_function_query_widget_online_cb),
    widget
  );

  g_signal_connect_after(
    G_OBJECT(priv->chm_file_chooser),
    "selection-changed",
    G_CALLBACK(c4s_function_query_widget_selection_changed_cb),
    widget
  );
}

static void 
c4s_function_query_widget_finalize(GObject* object)
{
  C4sFunctionQueryWidget* function_query_widget;
  C4sFunctionQueryWidgetPrivate* priv;

  function_query_widget = C4S_FUNCTION_QUERY_WIDGET(object);
  priv = C4S_FUNCTION_QUERY_WIDGET_PRIVATE(function_query_widget);

  G_OBJECT_CLASS(parent_class)->finalize(object);
}

static void
c4s_function_query_widget_set_property(GObject* object,
                                       guint prop_id,
                                       const GValue* value,
                                       GParamSpec* pspec)
{
  C4sFunctionQueryWidget* widget;
  C4sFunctionQueryWidgetPrivate* priv;

  g_return_if_fail(C4S_IS_FUNCTION_QUERY_WIDGET(object));

  widget = C4S_FUNCTION_QUERY_WIDGET(object);
  priv = C4S_FUNCTION_QUERY_WIDGET_PRIVATE(widget);

  switch(prop_id)
  {
  case PROP_ACTION:
    c4s_function_query_widget_set_action(widget, g_value_get_enum(value));
    break;
  case PROP_CHM:
    c4s_function_query_widget_set_chm(widget, g_value_get_string(value));
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
    break;
  }
}

static void
c4s_function_query_widget_get_property(GObject* object,
                                       guint prop_id,
                                       GValue* value,
                                       GParamSpec* pspec)
{
  C4sFunctionQueryWidget* widget;
  C4sFunctionQueryWidgetPrivate* priv;
  gchar* str;

  g_return_if_fail(C4S_IS_FUNCTION_QUERY_WIDGET(object));

  widget = C4S_FUNCTION_QUERY_WIDGET(object);
  priv = C4S_FUNCTION_QUERY_WIDGET_PRIVATE(widget);

  switch(prop_id)
  {
  case PROP_ACTION:
    g_value_set_enum(value, c4s_function_query_widget_get_action(widget));
    break;
  case PROP_CHM:
    str = c4s_function_query_widget_get_chm(widget);
    g_value_set_string(value, str);
    g_free(str);
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    break;
  }
}

static void
c4s_function_query_widget_class_init(C4sFunctionQueryWidgetClass* klass)
{
  GObjectClass* object_class = G_OBJECT_CLASS(klass);
  parent_class = GTK_VBOX_CLASS(g_type_class_peek_parent(klass));

  g_type_class_add_private(klass, sizeof(C4sFunctionQueryWidgetPrivate));

  object_class->finalize = c4s_function_query_widget_finalize;
  object_class->set_property = c4s_function_query_widget_set_property;
  object_class->get_property = c4s_function_query_widget_get_property;

  g_object_class_install_property(
    object_class,
    PROP_ACTION,
    g_param_spec_enum(
      "action",
      "Action",
      "Currently selected action",
      C4S_TYPE_FUNCTION_QUERY_ACTION,
      C4S_FUNCTION_QUERY_ACTION_ONLINE,
      G_PARAM_READWRITE
    )
  );
  
  g_object_class_install_property(
    object_class,
    PROP_CHM,
    g_param_spec_string(
      "chm",
      "CHM path",
      "Path to CHM documentation file",
      NULL,
      G_PARAM_READWRITE
    )
  );
  
  function_query_widget_signals[UPDATE] = g_signal_new(
    "update",
    G_OBJECT_CLASS_TYPE(object_class),
    G_SIGNAL_RUN_LAST,
    0,
    NULL, NULL,
    c4s_marshal_VOID__ENUM_STRING,
    G_TYPE_NONE,
    2,
    C4S_TYPE_FUNCTION_QUERY_ACTION | G_SIGNAL_TYPE_STATIC_SCOPE,
    G_TYPE_STRING
  );
}

GType
c4s_function_query_action_get_type(void)
{
  static GType etype = 0;
  if(etype == 0)
  {
    static const GEnumValue values[] = {
      {
        C4S_FUNCTION_QUERY_ACTION_CHM,
        "C4S_FUNCTION_QUERY_ACTION_CHM",
        "chm"
      }, {
        C4S_FUNCTION_QUERY_ACTION_ONLINE,
        "C4S_FUNCTION_QUERY_ACTION_ONLINE",
        "online"
      }, {
        0,
        NULL,
        NULL
      }
    };

    etype = g_enum_register_static("C4sFunctionQueryAction", values);
  }

  return etype;
}

GType
c4s_function_query_widget_get_type(void)
{
  static GType our_type = 0;

  if(our_type == 0)
  {
    static const GTypeInfo our_info = {
      sizeof(C4sFunctionQueryWidgetClass),
      (GBaseInitFunc)NULL,
      (GBaseFinalizeFunc)NULL,
      (GClassInitFunc)c4s_function_query_widget_class_init,
      NULL,
      NULL,
      sizeof(C4sFunctionQueryWidget),
      0,
      (GInstanceInitFunc)c4s_function_query_widget_init,
      NULL
    };

    our_type = g_type_register_static(
      GTK_TYPE_VBOX,
      "C4sFunctionQueryWidget",
      &our_info,
      0
    );
  }

  return our_type;
}

C4sFunctionQueryWidget*
c4s_function_query_widget_new(GtkWindow* parent)
{
  GObject* object;

  /* TODO: parent as construct only property, set in _constructor */
  object = g_object_new(
    C4S_TYPE_FUNCTION_QUERY_WIDGET,
    NULL
  );

  return C4S_FUNCTION_QUERY_WIDGET(object);
}

C4sFunctionQueryAction
c4s_function_query_widget_get_action(const C4sFunctionQueryWidget* widget)
{
  C4sFunctionQueryWidgetPrivate* priv;
  priv = C4S_FUNCTION_QUERY_WIDGET_PRIVATE(widget);

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(priv->radio_chm)))
    return C4S_FUNCTION_QUERY_ACTION_CHM;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(priv->radio_online)))
    return C4S_FUNCTION_QUERY_ACTION_ONLINE;

  g_assert_not_reached();
}

void
c4s_function_query_widget_set_action(C4sFunctionQueryWidget* widget,
                                     C4sFunctionQueryAction action)
{
  C4sFunctionQueryWidgetPrivate* priv;
  priv = C4S_FUNCTION_QUERY_WIDGET_PRIVATE(widget);

  switch(action)
  {
  case C4S_FUNCTION_QUERY_ACTION_CHM:
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(priv->radio_chm), TRUE);
    break;
  case C4S_FUNCTION_QUERY_ACTION_ONLINE:
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(priv->radio_online), TRUE);
    break;
  default:
    g_assert_not_reached();
    break;
  }
}

gchar*
c4s_function_query_widget_get_chm(const C4sFunctionQueryWidget* widget)
{
  C4sFunctionQueryWidgetPrivate* priv;
  priv = C4S_FUNCTION_QUERY_WIDGET_PRIVATE(widget);

  return gtk_file_chooser_get_filename(
    GTK_FILE_CHOOSER(priv->chm_file_chooser)
  );
}

void
c4s_function_query_widget_set_chm(C4sFunctionQueryWidget* widget,
                                  const gchar* path)
{
  C4sFunctionQueryWidgetPrivate* priv;
  priv = C4S_FUNCTION_QUERY_WIDGET_PRIVATE(widget);

  gtk_file_chooser_set_filename(
    GTK_FILE_CHOOSER(priv->chm_file_chooser),
    path
  );
}
