/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __C4S_HIGHLIGHTER_H__
#define __C4S_HIGHLIGHTER_H__

#include <c4scripter/c4s-enginefunctionlist.h>
#include <c4scripter/c4s-keywordlist.h>
#include <c4scripter/c4s-preferences.h>

#include <gtk/gtktextiter.h>

#include <glib-object.h>

G_BEGIN_DECLS

#define C4S_TYPE_HIGHLIGHTER			(c4s_highlighter_get_type())
#define C4S_HIGHLIGHTER(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), C4S_TYPE_HIGHLIGHTER, C4sHighlighter))
#define C4S_HIGHLIGHTER_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), C4S_TYPE_HIGHLIGHTER, C4sHighlighterClass))
#define C4S_IS_HIGHLIGHTER(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), C4S_TYPE_HIGHLIGHTER))
#define C4S_IS_HIGHLIGHTER_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), C4S_TYPE_HIGHLIGHTER))
#define C4S_HIGHLIGHTER_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), C4S_TYPE_HIGHLIGHTER, C4sHighlighterClass))

typedef struct _C4sHighlighterClass C4sHighlighterClass;
typedef struct _C4sHighlighter C4sHighlighter;

struct _C4sHighlighterClass
{
  GObjectClass parent_class;

  void(*highlighted)(C4sHighlighter*, GtkTextIter*, GtkTextIter*);
};

struct _C4sHighlighter
{
  GObject parent_instance;
};

GType
c4s_highlighter_get_type(void) G_GNUC_CONST;

C4sHighlighter*
c4s_highlighter_new(GtkTextBuffer* buffer,
                    C4sKeywordList* keywords,
                    C4sEngineFunctionList* engine_function_list,
                    C4sPreferences* preferences);

void
c4s_highlighter_forward_to_token(const C4sHighlighter* highlighter,
                                 GtkTextIter* iter);

void
c4s_highlighter_backward_to_token(const C4sHighlighter* highlighter,
                                  GtkTextIter* iter);

void
c4s_highlighter_forward_out_of_string(const C4sHighlighter* highlighter,
                                      GtkTextIter* iter);

void
c4s_highlighter_backward_out_of_string(const C4sHighlighter* highlighter,
                                       GtkTextIter* iter);

void
c4s_highlighter_forward_out_of_comment(const C4sHighlighter* highlighter,
                                       GtkTextIter* iter);

void
c4s_highlighter_backward_out_of_comment(const C4sHighlighter* highlighter,
                                        GtkTextIter* iter);

gboolean
c4s_highlighter_starts_string(const C4sHighlighter* highlighter,
                              const GtkTextIter* iter);

gboolean
c4s_highlighter_ends_string(const C4sHighlighter* highlighter,
                            const GtkTextIter* iter);

gboolean
c4s_highlighter_starts_comment(const C4sHighlighter* highlighter,
                               const GtkTextIter* iter);

gboolean
c4s_highlighter_ends_comment(const C4sHighlighter* highlighter,
                             const GtkTextIter* iter);

gboolean
c4s_highlighter_is_string(const C4sHighlighter* highlighter,
                          const GtkTextIter* iter);

gboolean
c4s_highlighter_is_comment(const C4sHighlighter* highlighter,
                           const GtkTextIter* iter);

/* !is_comment && !is_string */
gboolean
c4s_highlighter_is_code(const C4sHighlighter* highlighter,
                        const GtkTextIter* iter);

G_END_DECLS

#endif /* __C4S_HIGHLIGHTER_H__ */
