/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __C4S_HTTP_H__
#define __C4S_HTTP_H__

#include <c4scripter/c4s-tcp.h>

#include <glib-object.h>

G_BEGIN_DECLS

#define C4S_TYPE_HTTP_REQUEST			(c4s_http_request_get_type())
#define C4S_HTTP_REQUEST(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), C4S_TYPE_HTTP_REQUEST, C4sHttpRequest))
#define C4S_HTTP_REQUEST_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), C4S_TYPE_HTTP_REQUEST, C4sHttpRequestClass))
#define C4S_IS_HTTP_REQUEST(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), C4S_TYPE_HTTP_REQUEST))
#define C4S_IS_HTTP_REQUEST_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), C4S_TYPE_HTTP_REQUEST))
#define C4S_HTTP_REQUEST_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), C4S_TYPE_HTTP_REQUEST, C4sHttpRequestClass))

typedef struct _C4sHttpRequestClass C4sHttpRequestClass;
typedef struct _C4sHttpRequest C4sHttpRequest;

typedef enum _C4sHttpRequestError {
  C4S_HTTP_REQUEST_ERROR_CONNECTION, /* Connection problem */
  C4S_HTTP_REQUEST_ERROR_PROTOCOL, /* HTTP protocol error */
  C4S_HTTP_REQUEST_ERROR_DEFLATE, /* Compression error */
  C4S_HTTP_REQUEST_ERROR_FAILED
} C4sHttpRequestError;

typedef void(*C4sHttpErrorFunc)(C4sHttpRequest*, const GError*, gpointer);
typedef void(*C4sHttpDoneFunc)(C4sHttpRequest*, const gchar*, gsize, gpointer);

struct _C4sHttpRequestClass
{
  GObjectClass parent_class;
};

struct _C4sHttpRequest
{
  GObject parent_instance;
};

GType
c4s_http_request_get_type(void) G_GNUC_CONST;

C4sHttpRequest*
c4s_http_request_new(C4sTcpConnection* conn,
                     const gchar* request_file,
                     C4sHttpDoneFunc done_func,
                     C4sHttpErrorFunc error_func,
                     gpointer user_data);

const gchar*
c4s_http_request_get_file(const C4sHttpRequest* request);

G_END_DECLS

#endif /* __C4S_HTTP_H__ */
